package ch.unige.solidify.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyHttpErrorException;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.CompositeResource;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.SubResourceWithRelationContainer;

public abstract class Relation2TiersController<T extends Resource<T> & SubResourceWithRelationContainer, V extends Resource<V>, Z extends CompositeResource<Z, ?>>
        extends RelationController<T, V, Z> {

  @PostMapping
  public HttpEntity<?> create(@PathVariable String parentid, @RequestBody String[] ids) {
    final T parentItem = this.itemService.findOne(parentid);
    final List<V> list = new ArrayList<>();
    for (final String i : ids) {
      final V childItem = this.childItemService.findOne(i);
      if (parentItem.addItem(childItem)) {
        list.add(childItem);
      }
    }
    if (!list.isEmpty()) {
      this.itemService.save(parentItem);
      return new ResponseEntity<>(list, HttpStatus.CREATED);
    }
    return new ResponseEntity<>(HttpStatus.OK);
  }

  @DeleteMapping(value = SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return this.deleteList(parentid, new String[] { id });
  }

  @DeleteMapping
  public HttpStatus deleteList(@PathVariable String parentid,
          @RequestBody(required = false) String[] ids) {
    final T parentItem = this.itemService.findOne(parentid);
    final List<V> childList = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid);
    if (childList.isEmpty()) {
      throw new SolidifyHttpErrorException(HttpStatus.GONE, "Parent item " + parentid + " gone");
    }
    final List<Z> list = new ArrayList<>();
    for (final V childItem : childList) {
      if (this.isInList(ids, childItem.getResId())) {
        final Z childToDelete = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
                this.getMainResourceClass(), parentid, this.getSubResourceClass(),
                childItem.getResId());
        if (childToDelete != null && parentItem.removeItem(childToDelete)) {
          list.add(childToDelete);
        }
      }
    }
    if (!list.isEmpty()) {
      this.itemService.save(parentItem);
    }
    return HttpStatus.OK;
  }

  @GetMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<?> get(@PathVariable String parentid, @PathVariable String id) {
    final T parentItem = this.itemService.findOne(parentid);
    final V childItem = this.childItemService.findOne(id);
    final Z childDetail = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentItem.getResId(), this.getSubResourceClass(), id);
    if (childDetail == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    final Object newChild = parentItem.getDetails(childItem, childDetail);

    this.addLinks(parentid, (V) newChild);
    return new ResponseEntity<>(newChild, HttpStatus.OK);
  }

  @Override
  @SuppressWarnings("unchecked")
  @GetMapping
  public HttpEntity<Collection<?>> list(@PathVariable String parentid, Pageable pageable) {
    final T mainResourceItem = this.itemService.findOne(parentid);
    final Page<V> childList = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid, pageable);
    final List<Object> newList = new ArrayList<>();
    for (final V childItem : childList) {
      final Z childDetail = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
              this.getMainResourceClass(), mainResourceItem.getResId(), this.getSubResourceClass(),
              childItem.getResId());
      final Object newChild = mainResourceItem.getDetails(childItem, childDetail);
      this.addLinks(parentid, (V) newChild);
      newList.add(newChild);
    }

    final Collection<?> collection = this.processPageAndLinks(newList, childList, parentid, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @PatchMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<Z> update(@PathVariable String parentid, @PathVariable String id, @RequestBody Z newItem) {
    final T parentItem = this.itemService.findOne(parentid);
    final Z oldItem = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid, this.getSubResourceClass(), id);
    if (oldItem == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (oldItem.update(newItem)) {
      this.itemService.save(parentItem);
    }
    return new ResponseEntity<>(HttpStatus.OK);
  }

}
