package ch.unige.solidify.repository;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.repository.NoRepositoryBean;

@NoRepositoryBean
public interface SolidifyRepositoryNoSql<T> {
  public boolean delete(T t);

  // CRUD functions
  public List<T> findAll(String indexName, T search);

  public default Page<T> findAll(String indexName, T search, Pageable pageable) {
    final List<T> list = this.findAll(indexName, search);
    final long start = pageable.getOffset();
    final long end = (start + pageable.getPageSize()) > list.size() ? list.size()
            : (start + pageable.getPageSize());
    return new PageImpl<>(list.subList((int) start, (int) end), pageable, list.size());
  }

  public T findOne(String indexName, String id);

  public T save(T t);

  // Default Functions
  public default Page<T> search(String indexName, String query, Pageable pageable) {
    return this.search(indexName, query, null, null, null, null, pageable);
  }

  public default Page<T> search(String indexName, String query, String set, String field,
          String from, String to, Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  public default Page<T> search(String indexName, String query, String set, String field,
          String from, String to, String filter, Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  public default Page<T> searchRange(String indexName, String field, String from, String to,
          Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  // Define Database name ~ index name
  public void setIndex(String indexName);

  public T update(T t);
}
