package ch.unige.solidify.validation;

import java.util.ArrayList;
import java.util.List;

import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;

import com.fasterxml.jackson.annotation.JsonInclude;

public class ValidationError {

  @JsonInclude(JsonInclude.Include.NON_EMPTY)
  private final List<FieldValidationError> fieldErrors = new ArrayList<>();

  private String mainErrorMessage;

  /****************/

  public ValidationError() {

  }

  public ValidationError(BindingResult errors) {

    final String errorMessage = "Validation failed on " + errors.getObjectName() + ": "
            + errors.getErrorCount() + " error(s)";
    this.setMainErrorMessage(errorMessage);

    this.addErrors(errors);
  }

  public ValidationError(String errorMessage) {
    this.setMainErrorMessage(errorMessage);
  }

  public ValidationError(String errorMessage, BindingResult errors) {
    this.setMainErrorMessage(errorMessage);
    this.addErrors(errors);
  }

  public void addError(ObjectError error) {

    final String fieldName = error instanceof FieldError ? ((FieldError) error).getField() : error.getObjectName();
    final String errorMessage = error.getDefaultMessage();

    this.addError(fieldName, errorMessage);
  }

  public void addError(String fieldName, String errorMessage) {

    FieldValidationError fieldError = this.getFieldValidationError(fieldName);
    if (fieldError == null) {

      fieldError = new FieldValidationError(fieldName, errorMessage);
      this.fieldErrors.add(fieldError);

    } else {
      fieldError.addErrorMessage(errorMessage);
    }
  }

  /****************/

  public void addErrors(BindingResult errors) {

    final List<ObjectError> errorsList = errors.getAllErrors();

    for (final ObjectError error : errorsList) {
      this.addError(error);
    }
  }

  public void clearErrors() {
    this.fieldErrors.clear();
  }

  public List<FieldValidationError> getErrors() {
    return this.fieldErrors;
  }

  public FieldValidationError getFieldValidationError(String fieldname) {

    for (final FieldValidationError fieldError : this.fieldErrors) {
      if (fieldError.getFieldName().equals(fieldname)) {
        return fieldError;
      }
    }

    return null;
  }

  public String getMainErrorMessage() {
    return this.mainErrorMessage;
  }

  public boolean hasError() {
    return !this.fieldErrors.isEmpty();
  }

  public void setMainErrorMessage(String mainErrorMessage) {
    this.mainErrorMessage = mainErrorMessage;
  }
}
